#!/usr/bin/env python3
# coding: utf-8

import csv
import matplotlib.pyplot as plt
import numpy as np
import netCDF4 as nc
import os

import cartopy.crs as ccrs
import cartopy.feature as cfeature
import matplotlib.gridspec as gridspec

import metpy.calc as mpcalc
import cmocean


titles = ['Tran_Glac1d', 'Tran_Paleomist', 'Tran_PISMindex', ]
ticks = np.array([ 2000, 5000, 8000, 11000,  14000, 17000, ])
ticks2 =  (38000-ticks)/1000
ticks2 = ticks2.astype(int)



colorline = 'firebrick'

########################
## in kyr
years = np.arange(0,38001,100)


projection = ccrs.NorthPolarStereo(central_longitude= -45,true_scale_latitude=70.,globe=None)
print('pro4_params: ',projection.proj4_params)

################ start figure
fig = plt.figure(figsize=(14,16))
fig.subplots_adjust(left=0.08,right=0.82, top=0.98, bottom=0.05)

# Use gridspec to help size elements of plot; small top plot and big bottom plot
gs = gridspec.GridSpec(nrows=6, ncols=3, height_ratios=[2, 3, 3, 3,3,3], hspace=0.1, wspace=0.07)


##################################################################
## plot line 1
##################################################################

fins1 = ['../Figure1/glac0.3_thk_topg_1_notime_select.nc',
       '../Figure1/pma10.3_thk_topg_1_notime_select.nc',
       '../Figure1/pism0.3_thk_topg_1_notime_select.nc',
       ]

fins2 = ['glac0.3_thk_multi.nc',
        'pma10.3_thk_multi.nc',
        'pism0.3_thk_multi.nc', ]

finsarea = ['./area_change/glac0.3_area.nc',
            './area_change/pma10.3_area.nc',
            './area_change/pism0.3_area.nc']

colorarea = plt.cm.gray
levelarea = np.arange(40, 90, 2.)

nfiles = len(fins1)
nt = 10


for i in range(0,nfiles):
    # Top plot for geographic reference (makes small map)
    ax1 = fig.add_subplot(gs[0, i], projection=ccrs.PlateCarree())
    ax1.set_extent([-140, -50, 30, 70], ccrs.PlateCarree())

    if i == 0:
        ax1.set_yticks([40,50, 60])
        ax1.set_yticklabels([u'40\N{DEGREE SIGN}N', u'50\N{DEGREE SIGN}N', u'60\N{DEGREE SIGN}N'])
    else:
        ax1.set_yticks([ ])
        ax1.set_yticklabels([])

    ax1.set_xticks([  -120, -100, -80, -60 ])
    ax1.set_xticklabels([ u'120\N{DEGREE SIGN}E',u'100\N{DEGREE SIGN}E',u'80\N{DEGREE SIGN}E',u'60\N{DEGREE SIGN}E'])
    ax1.tick_params(  labelsize=13)
    ax1.coastlines()

    # Set some titles
    plt.title('Ice margin', loc='right', fontsize=13)
    plt.title('    '+titles[i], loc='left', font='serif', fontsize=15, horizontalalignment='left')

    ################### ice sheet initial #########################
    with nc.Dataset(fins1[i]) as ff:
        data1 = ff.variables['thk'][:]
        x = ff.variables['x'][:]
        y = ff.variables['y'][:]
        topg = ff.variables['topg'][:]
        usurf = topg + data1

        new_extent = [np.min(x), np.max(x), np.max(y), np.min(y)]

        cc = ax1.contour(x, y, data1,  transform=projection,extent=new_extent, colors=colorline, levels=[1.],linewidths=1.2 )
        # print(cc.levels)
        fmt = {}
        fmt[1.] = '38 ka'
        plt.clabel(cc,  fontsize=10, fmt=fmt ,colors=colorline,inline_spacing=20)


    ######## ice sheet multiple time steps ##########################
    with nc.Dataset(fins2[i]) as ff:
        thk = ff.variables['thk'][:]

    cs3 = ax1.contour(x, y, thk[0,:,:], transform=projection,extent=new_extent, colors=colorline, levels=[0.],linewidths=1.5 , linestyles='--')
    fmt = {}
    fmt[0.] = '28 ka'
    plt.clabel(cs3,  fontsize=10, fmt=fmt ,  colors=colorline,inline_spacing=20)

    cs3 = ax1.contour(x, y, thk[1,:,:], transform=projection,extent=new_extent, colors=colorline, levels=[0.],linewidths=1.5 , linestyles=':')
    fmt = {}
    fmt[0.] = '21 ka'
    plt.clabel(cs3,  fontsize=10, fmt=fmt ,  colors=colorline,inline_spacing=20)

    ######## ice sheet reconstruction ##########################
    with nc.Dataset('../Figure1/paleomist_thk_notime_select.nc','r') as ff:
        thkr = ff.variables['thk'][:,:]
        ax1.contour(x,y, thkr,  transform=projection,extent=new_extent, colors='khaki', levels=[1.],linewidths=1.5 , linestyles='-')





lon1 = 220
lon2 = 310

#
# ##################################################################
# ## plot line 2
# ##################################################################
fins = ['./meri_smbpism/glac0.3_smb_annual_mermean_nais_annualmean.nc',
        './meri_smbpism/pma10.3_smb_annual_mermean_nais_annualmean.nc',
        './meri_smbpism/pism0.3_smb_annual_mermean_nais_annualmean.nc']

maxval = 1.2
minval = -1.2
breaks = np.linspace(minval, maxval, 61)
color = cmocean.cm.phase
color = plt.cm.RdYlBu

for i in range(0,nfiles):
    with nc.Dataset(fins[i]) as ff:
        data = ff.variables['tendency_of_ice_amount_due_to_surface_mass_flux'][:,0,:]
        lons = ff.variables['lon'][:]
        # kg/m2year -> m/year ice equivalent
        data = data /917.


    inds = np.where((lons >lon1) & (lons<lon2))
    lons = lons[inds]
    data = data[:,inds][:,0,:]


    # Top plot for geographic reference (makes small map)
    ax = fig.add_subplot(gs[1, i])

    ax.invert_yaxis()  # Reverse the time order to do oldest first
    if i == 0:
        ax.set_ylabel('Time (ka)', fontsize=18)
        ax.set_yticks(ticks)
        ax.set_yticklabels(ticks2)
        ax.tick_params(axis='y',  labelsize=13)
    elif i == 1:
        ax.set_yticks([])
    else:
        ax.set_yticks(ticks)
        ax.set_yticklabels(ticks2)
        ax.tick_params(axis='y', left=False, labelleft=False, right=True, labelright=True, labelsize=13)



    cf = ax.contourf(lons, years[:data.shape[0]], data , levels=breaks, vmin=minval,vmax=maxval, cmap=color, extend='both')

    ax.set_xticks([ ])
    ax.tick_params(axis='x',  labelsize=13)


    with nc.Dataset(finsarea[i]) as ff:
        south = ff.variables['southward'][:]
        #print(south.shape)
        south = south[:,inds][:,0,:]
        #print(np.max(south))

    cc = ax.contour(lons, years[:data.shape[0]], south, levels=levelarea, cmap=colorarea,) # colors='black')
    plt.clabel(cc,  fontsize=13)

cax = plt.axes([0.88,0.72,0.012,0.15])
cbar = plt.colorbar(cf, cax=cax,orientation='vertical',fraction=0.05,pad=0.03, label='Precipitation (m/year)', extendrect=True)
cbar.set_label('SMB (m/year)\n Annual Mean',fontsize=15)
cbar.set_ticks(np.arange(minval, maxval+0.01, 0.3))
cbar.ax.tick_params(labelsize=13)





##################################################################
## plot line 3
##################################################################

fins = ['./meri_smb/glac0.3_smb_annual_mermean_nais_summer_positive.nc',
        './meri_smb/pma10.3_smb_annual_mermean_nais_summer_positive.nc',
        './meri_smb/pism0.3_smb_annual_mermean_nais_summer_positive.nc']

maxval = 1.8
minval = 0.
breaks = np.linspace(minval, maxval, 61)
color = cmocean.cm.phase
color = plt.cm.Purples

for i in range(0,nfiles):
    with nc.Dataset(fins[i]) as ff:
        data = ff.variables['SMB'][:,0,:] * 365 * 24 * 3600/917.    #kg/m2s ->  m/year    with ice density 917kg/m3
        lons = ff.variables['lon'][:]


    inds = np.where((lons >lon1) & (lons<lon2))
    lons = lons[inds]
    data = data[:,inds][:,0,:]


    # Top plot for geographic reference (makes small map)
    ax = fig.add_subplot(gs[2, i])

    ax.invert_yaxis()  # Reverse the time order to do oldest first
    if i == 0:
        ax.set_ylabel('Time (ka)', fontsize=18)
        ax.set_yticks(ticks)
        ax.set_yticklabels(ticks2)
        ax.tick_params(axis='y',  labelsize=13)
    elif i == 1:
        ax.set_yticks([])
    else:
        ax.set_yticks(ticks)
        ax.set_yticklabels(ticks2)
        ax.tick_params(axis='y', left=False, labelleft=False, right=True, labelright=True, labelsize=13)



    cf = ax.contourf(lons, years[:data.shape[0]], data , levels=breaks, vmin=minval,vmax=maxval, cmap=color, extend='both')
    ax.set_xticks([])


    with nc.Dataset(finsarea[i]) as ff:
        south = ff.variables['southward'][:]
        #print(south.shape)
        south = south[:,inds][:,0,:]
        #print(np.max(south))

    cc = ax.contour(lons, years[:data.shape[0]], south, levels=levelarea, cmap=colorarea,) # colors='black')
    plt.clabel(cc,  fontsize=13)

    cax = plt.axes([0.88,0.55,0.012,0.15])
    cbar = plt.colorbar(cf, cax=cax,orientation='vertical',fraction=0.05,pad=0.03,  extendrect=True)
    cbar.set_label('SMB (m/year)\nAccumulation zone\nSummer',fontsize=15)
    cbar.set_ticks(np.arange(minval,maxval+0.01, 0.3))
    cbar.ax.tick_params(labelsize=13)



##################################################################
## plot line 4
##################################################################

fins = ['./meri_smb/glac0.3_smb_annual_mermean_nais_summer_negative.nc',
        './meri_smb/pma10.3_smb_annual_mermean_nais_summer_negative.nc',
        './meri_smb/pism0.3_smb_annual_mermean_nais_summer_negative.nc']

maxval = 0
minval = -12
breaks = np.linspace(minval, maxval, 81)
color = cmocean.cm.phase
color = plt.cm.YlOrBr_r

for i in range(0,nfiles):
    with nc.Dataset(fins[i]) as ff:
        data = ff.variables['SMB'][:,0,:] * 365 * 24 * 3600/917.   #kg/m2s ->  m/year    with ice density 917kg/m3
        lons = ff.variables['lon'][:]


    inds = np.where((lons >lon1) & (lons<lon2))
    lons = lons[inds]
    data = data[:,inds][:,0,:]



    # Top plot for geographic reference (makes small map)
    ax = fig.add_subplot(gs[3, i])

    ax.invert_yaxis()  # Reverse the time order to do oldest first
    if i == 0:
        ax.set_ylabel('Time (ka)', fontsize=18)
        ax.set_yticks(ticks)
        ax.set_yticklabels(ticks2)
        ax.tick_params(axis='y',  labelsize=13)
    elif i == 1:
        ax.set_yticks([])
    else:
        ax.set_yticks(ticks)
        ax.set_yticklabels(ticks2)
        ax.tick_params(axis='y', left=False, labelleft=False, right=True, labelright=True, labelsize=13)



    cf = ax.contourf(lons, years[:data.shape[0]], data , levels=breaks, vmin=minval,vmax=maxval, cmap=color, extend='both')
    ax.set_xticks([])


    with nc.Dataset(finsarea[i]) as ff:
        south = ff.variables['southward'][:]
        south = south[:,inds][:,0,:]

    cc = ax.contour(lons, years[:data.shape[0]], south, levels=levelarea, cmap=colorarea,) # colors='black')
    plt.clabel(cc,  fontsize=13)

    cax = plt.axes([0.88,0.38,0.012,0.15])
    cbar = plt.colorbar(cf, cax=cax,orientation='vertical',fraction=0.05,pad=0.03, extendrect=True)
    cbar.set_label('SMB (m/year)\nAblation zone\nSummer',fontsize=15)
    cbar.set_ticks(np.arange(minval,maxval+0.01, 2))
    cbar.ax.tick_params(labelsize=13)





##################################################################
## plot line 5
##################################################################


fins = ['./meri_snow/glac0.3_snow_mermean_nais_summer.nc',
        './meri_snow/pma10.3_snow_mermean_nais_summer.nc',
        './meri_snow/pism0.3_snow_mermean_nais_summer.nc']

maxval = 1.8
minval = 0  # -3e-6
breaks = np.linspace(minval, maxval, 61)
color = cmocean.cm.phase  #balance_r #phase #dense #matter   #RdBu_r  #twilight
color = plt.cm.terrain_r

for i in range(0,nfiles):
    with nc.Dataset(fins[i]) as ff:
        data = ff.variables['SF'][:,0,:]
        lons = ff.variables['lon'][:]
        # kg/m2s -> m/year
        data = data  * 365 * 24 * 3600/917.



    inds = np.where((lons >lon1) & (lons<lon2))
    lons = lons[inds]
    data = data[:,inds][:,0,:]


    # Top plot for geographic reference (makes small map)
    ax = fig.add_subplot(gs[4, i])

    ax.invert_yaxis()  # Reverse the time order to do oldest first
    if i == 0:
        ax.set_ylabel('Time (ka)', fontsize=18)
        ax.set_yticks(ticks)
        ax.set_yticklabels(ticks2)
        ax.tick_params(axis='y',  labelsize=13)
    elif i == 1:
        ax.set_yticks([])
    else:
        ax.set_yticks(ticks)
        ax.set_yticklabels(ticks2)
        ax.tick_params(axis='y', left=False, labelleft=False, right=True, labelright=True, labelsize=13)



    cf = ax.contourf(lons, years[:data.shape[0]], data , levels=breaks, vmin=minval,vmax=maxval, cmap=color, extend='both')

    ax.set_xticks([ 220, 240, 260, 280, 300 ])
    ax.set_xticklabels([ u'140\N{DEGREE SIGN}E',u'120\N{DEGREE SIGN}E',u'100\N{DEGREE SIGN}E',u'80\N{DEGREE SIGN}E',u'60\N{DEGREE SIGN}E'])
    ax.tick_params(axis='x',  labelsize=13)

    with nc.Dataset(finsarea[i]) as ff:
        south = ff.variables['southward'][:]
        south = south[:,inds][:,0,:]

    cc = ax.contour(lons, years[:data.shape[0]], south, levels=levelarea, cmap=colorarea,) # colors='black')
    plt.clabel(cc,  fontsize=13)


    cax = plt.axes([0.88,0.21,0.012,0.15])
    cbar = plt.colorbar(cf, cax=cax,orientation='vertical',fraction=0.05,pad=0.03,  extendrect=True)
    cbar.set_label('Snow (m/year)\nSummer',fontsize=15)
    cbar.set_ticks(np.arange(minval, maxval+0.01, 0.3))
    cbar.ax.tick_params(labelsize=13)



plt.savefig('Figure3.png')
plt.show(block=False)
exit()
